@@ -811,6 +811,8 @@ func (b *Build) run(ctx context.Context, trace JobTrace, executor Executor) (err
 		buildFinish <- b.executeScript(runContext, trace, executor)
 	}()
 
+	defer b.ensureFinishedAt()
+
 	// Wait for signals: cancel, timeout, abort or finish
 	b.Log().Debugln("Waiting for signals...")
 	select {
@@ -853,8 +855,6 @@ func (b *Build) run(ctx context.Context, trace JobTrace, executor Executor) (err
 	runCancel()
 	b.waitForBuildFinish(buildFinish, WaitForBuildFinishTimeout)
 
-	b.ensureFinishedAt()
-
 	return err
 }
 
