@@ -8,6 +8,8 @@ import (
 	"fmt"
 	"io"
 	"slices"
+	"sort"
+	"strconv"
 	"strings"
 	"time"
 
@@ -117,6 +119,7 @@ var (
 		ImagePullFailure,
 		UnknownFailure,
 		ConfigurationError,
+		JobCanceled,
 	}
 
 	// failureReasonsCompatibilityMap contains a mapping of new failure reasons
@@ -390,6 +393,8 @@ func (eo *executorOptions) validate(data []byte, supportedOptions []string, exec
 		}
 	}
 	if len(notSupported) != 0 {
+		sort.Strings(supportedOptions)
+
 		return &UnsuportedExecutorOptionsError{
 			executor:           executor,
 			section:            section,
@@ -406,7 +411,8 @@ func (eo *executorOptions) UnsupportedOptions() error {
 }
 
 var SupportedExecutorOptions = map[string][]string{
-	"docker": {"platform", "user"},
+	"docker":     {"platform", "user"},
+	"kubernetes": {"user"},
 }
 
 type (
@@ -415,9 +421,14 @@ type (
 		Platform string `json:"platform"`
 		User     string `json:"user"`
 	}
+	ImageKubernetesOptions struct {
+		executorOptions
+		User string `json:"user"`
+	}
 	ImageExecutorOptions struct {
 		executorOptions
-		Docker ImageDockerOptions `json:"docker,omitempty"`
+		Docker     ImageDockerOptions     `json:"docker,omitempty"`
+		Kubernetes ImageKubernetesOptions `json:"kubernetes,omitempty"`
 	}
 )
 
@@ -442,6 +453,42 @@ func (ido *ImageDockerOptions) UnmarshalJSON(data []byte) error {
 	return nil
 }
 
+func (iko *ImageKubernetesOptions) UnmarshalJSON(data []byte) error {
+	type imageKubernetesOptions ImageKubernetesOptions
+	inner := imageKubernetesOptions{}
+	if err := json.Unmarshal(data, &inner); err != nil {
+		return err
+	}
+	*iko = ImageKubernetesOptions(inner)
+
+	// call validate after json.Unmarshal so the former handles bad json.
+	iko.unsupportedOptions = iko.validate(data, SupportedExecutorOptions["kubernetes"], "kubernetes executor", "image")
+	return nil
+}
+
+func (iko *ImageKubernetesOptions) GetUIDGID() (int64, int64, error) {
+	if iko.User == "" {
+		return 0, 0, nil
+	}
+
+	user, group, ok := strings.Cut(iko.User, ":")
+
+	uid, err := strconv.ParseInt(user, 10, 64)
+	if err != nil {
+		return 0, 0, fmt.Errorf("failed to parse UID %w", err)
+	}
+
+	var gid int64
+	if ok {
+		gid, err = strconv.ParseInt(group, 10, 64)
+		if err != nil {
+			return 0, 0, fmt.Errorf("failed to parse GID %w", err)
+		}
+	}
+
+	return uid, gid, err
+}
+
 func (ieo *ImageExecutorOptions) UnmarshalJSON(data []byte) error {
 	type imageExecutorOptions ImageExecutorOptions
 	inner := imageExecutorOptions{}
@@ -456,7 +503,11 @@ func (ieo *ImageExecutorOptions) UnmarshalJSON(data []byte) error {
 }
 
 func (ieo *ImageExecutorOptions) UnsupportedOptions() error {
-	return errors.Join(ieo.executorOptions.UnsupportedOptions(), ieo.Docker.UnsupportedOptions())
+	return errors.Join(
+		ieo.executorOptions.UnsupportedOptions(),
+		ieo.Docker.UnsupportedOptions(),
+		ieo.Kubernetes.UnsupportedOptions(),
+	)
 }
 
 type Image struct {
@@ -1051,7 +1102,6 @@ type SupportedFailureReasonMapper interface {
 	Map(fr JobFailureReason) JobFailureReason
 }
 
-//go:generate mockery --name=JobTrace --inpackage
 type JobTrace interface {
 	io.Writer
 	Success() error
@@ -1087,7 +1137,6 @@ func NewPatchTraceResult(sentOffset int, state PatchState, newUpdateInterval int
 	}
 }
 
-//go:generate mockery --name=Network --inpackage
 type Network interface {
 	SetConnectionMaxAge(time.Duration)
 	RegisterRunner(config RunnerCredentials, parameters RegisterRunnerParameters) *RegisterRunnerResponse
