@@ -196,6 +196,7 @@ type doJSONParams struct {
 	method      string
 	uri         string
 	statusCode  int
+	headers     http.Header
 	request     interface{}
 	response    interface{}
 }
@@ -224,6 +225,7 @@ func (n *GitLabClient) doMeasuredJSON(
 			params.method,
 			params.uri,
 			params.statusCode,
+			params.headers,
 			params.request,
 			params.response,
 		)
@@ -242,76 +244,39 @@ func (n *GitLabClient) doMeasuredJSON(
 	return result, statusText, httpResponse
 }
 
+// Create a PRIVATE-TOKEN http header for the specified private access token (pat).
+func PrivateTokenHeader(pat string) http.Header {
+	headers := http.Header{}
+	if pat != "" {
+		headers.Set(common.PrivateToken, pat)
+	}
+	return headers
+}
+
+// Create a JOB-TOKEN http header for the specified job token.
+func JobTokenHeader(jobToken string) http.Header {
+	headers := http.Header{}
+	if jobToken != "" {
+		headers.Set(common.JobToken, jobToken)
+	}
+	return headers
+}
+
+// Create a RUNNER-TOKEN http header for the specified job token.
+func RunnerTokenHeader(runnerToken string) http.Header {
+	headers := http.Header{}
+	if runnerToken != "" {
+		headers.Set(common.RunnerToken, runnerToken)
+	}
+	return headers
+}
+
 func (n *GitLabClient) doJSON(
 	ctx context.Context,
 	credentials requestCredentials,
 	method, uri string,
 	statusCode int,
-	request interface{},
-	response interface{},
-) (int, string, *http.Response) {
-	return n.doJSONWithPAT(ctx, credentials, method, uri, statusCode, "", request, response)
-}
-
-type doJSONWithPATParams struct {
-	credentials requestCredentials
-	method      string
-	uri         string
-	statusCode  int
-	pat         string
-	request     interface{}
-	response    interface{}
-}
-
-// doMeasuredJSONWithPAT is a decorator that adds metrics measurements through
-// n.apiRequestsCollector to the doJSONWithPAT() call
-func (n *GitLabClient) doMeasuredJSONWithPAT(
-	ctx context.Context,
-	log logrus.FieldLogger,
-	runnerID string,
-	systemID string,
-	endpoint apiEndpoint,
-	params doJSONWithPATParams,
-) (int, string, *http.Response) {
-	var result int
-	var statusText string
-	var httpResponse *http.Response
-
-	fn := func() int {
-		// Response body is handled after doJSONWithPATParams() decorator call
-		// Linting violation here is a false-positive.
-		// nolint:bodyclose
-		result, statusText, httpResponse = n.doJSONWithPAT(
-			ctx,
-			params.credentials,
-			params.method,
-			params.uri,
-			params.statusCode,
-			params.pat,
-			params.request,
-			params.response,
-		)
-
-		return result
-	}
-
-	n.apiRequestsCollector.Observe(
-		log,
-		runnerID,
-		systemID,
-		endpoint,
-		fn,
-	)
-
-	return result, statusText, httpResponse
-}
-
-func (n *GitLabClient) doJSONWithPAT(
-	ctx context.Context,
-	credentials requestCredentials,
-	method, uri string,
-	statusCode int,
-	pat string,
+	headers http.Header,
 	request interface{},
 	response interface{},
 ) (int, string, *http.Response) {
@@ -320,7 +285,7 @@ func (n *GitLabClient) doJSONWithPAT(
 		return clientError, err.Error(), nil
 	}
 
-	return c.doJSONWithPAT(ctx, uri, method, statusCode, pat, request, response)
+	return c.doJSON(ctx, uri, method, statusCode, headers, request, response)
 }
 
 func (n *GitLabClient) getResponseTLSData(
@@ -352,13 +317,13 @@ func (n *GitLabClient) RegisterRunner(
 	}
 
 	var response common.RegisterRunnerResponse
-	result, statusText, resp := n.doJSONWithPAT(
+	result, statusText, resp := n.doJSON(
 		context.Background(),
 		&runner,
 		http.MethodPost,
 		"runners",
 		http.StatusCreated,
-		runner.Token,
+		RunnerTokenHeader(runner.Token),
 		&request,
 		&response,
 	)
@@ -387,25 +352,25 @@ func (n *GitLabClient) VerifyRunner(runner common.RunnerCredentials, systemID st
 	}
 
 	var response common.VerifyRunnerResponse
-	result, statusText, resp := n.doJSONWithPAT(
+	result, statusText, resp := n.doJSON(
 		context.Background(),
 		&runner,
 		http.MethodPost,
 		"runners/verify",
 		http.StatusOK,
-		runner.Token,
+		RunnerTokenHeader(runner.Token),
 		&request,
 		&response,
 	)
 	if result == -1 {
 		// if server is not able to return JSON, let's try plain text (the legacy response format)
-		result, statusText, resp = n.doJSONWithPAT(
+		result, statusText, resp = n.doJSON(
 			context.Background(),
 			&runner,
 			http.MethodPost,
 			"runners/verify",
 			http.StatusOK,
-			runner.Token,
+			RunnerTokenHeader(runner.Token),
 			&request,
 			nil,
 		)
@@ -442,13 +407,13 @@ func (n *GitLabClient) UnregisterRunner(runner common.RunnerCredentials) bool {
 		Token: runner.Token,
 	}
 
-	result, statusText, resp := n.doJSONWithPAT(
+	result, statusText, resp := n.doJSON(
 		context.Background(),
 		&runner,
 		http.MethodDelete,
 		"runners",
 		http.StatusNoContent,
-		runner.Token,
+		RunnerTokenHeader(runner.Token),
 		&request,
 		nil,
 	)
@@ -477,13 +442,13 @@ func (n *GitLabClient) UnregisterRunnerManager(runner common.RunnerCredentials,
 		SystemID: systemID,
 	}
 
-	result, statusText, resp := n.doJSONWithPAT(
+	result, statusText, resp := n.doJSON(
 		context.Background(),
 		&runner,
 		http.MethodDelete,
 		"runners/managers",
 		http.StatusNoContent,
-		runner.Token,
+		RunnerTokenHeader(runner.Token),
 		&request,
 		nil,
 	)
@@ -533,18 +498,18 @@ func (n *GitLabClient) resetToken(
 
 	var response common.ResetTokenResponse
 
-	result, statusText, resp := n.doMeasuredJSONWithPAT(
+	result, statusText, resp := n.doMeasuredJSON(
 		context.Background(),
 		runner.Log(),
 		runner.ShortDescription(),
 		systemID,
 		apiEndpointResetToken,
-		doJSONWithPATParams{
+		doJSONParams{
 			credentials: &runner,
 			method:      http.MethodPost,
 			uri:         uri,
 			statusCode:  http.StatusCreated,
-			pat:         pat,
+			headers:     PrivateTokenHeader(pat),
 			request:     request,
 			response:    &response,
 		},
@@ -603,18 +568,18 @@ func (n *GitLabClient) RequestJob(
 	var response common.JobResponse
 
 	//nolint:bodyclose
-	result, statusText, httpResponse := n.doMeasuredJSONWithPAT(
+	result, statusText, httpResponse := n.doMeasuredJSON(
 		ctx,
 		config.Log(),
 		config.RunnerCredentials.ShortDescription(),
 		config.SystemIDState.GetSystemID(),
 		apiEndpointRequestJob,
-		doJSONWithPATParams{
+		doJSONParams{
 			credentials: &config.RunnerCredentials,
 			method:      http.MethodPost,
 			uri:         "jobs/request",
 			statusCode:  http.StatusCreated,
-			pat:         config.Token,
+			headers:     RunnerTokenHeader(config.Token),
 			request:     &request, response: &response,
 		},
 	)
@@ -674,18 +639,18 @@ func (n *GitLabClient) UpdateJob(
 	log.Info("Updating job...")
 
 	//nolint:bodyclose
-	statusCode, statusText, response := n.doMeasuredJSONWithPAT(
+	statusCode, statusText, response := n.doMeasuredJSON(
 		context.Background(),
 		config.Log(),
 		config.RunnerCredentials.ShortDescription(),
 		config.SystemIDState.GetSystemID(),
 		apiEndpointUpdateJob,
-		doJSONWithPATParams{
+		doJSONParams{
 			credentials: &config.RunnerCredentials,
 			method:      http.MethodPut,
 			uri:         fmt.Sprintf("jobs/%d", jobInfo.ID),
 			statusCode:  http.StatusOK,
-			pat:         jobCredentials.Token,
+			headers:     JobTokenHeader(jobCredentials.Token),
 			request:     &request,
 			response:    nil,
 		},
@@ -763,9 +728,8 @@ func (n *GitLabClient) PatchTrace(
 	endOffset := startOffset + len(content)
 	contentRange := fmt.Sprintf("%d-%d", startOffset, endOffset-1)
 
-	headers := make(http.Header)
+	headers := JobTokenHeader(jobCredentials.Token)
 	headers.Set("Content-Range", contentRange)
-	headers.Set("JOB-TOKEN", jobCredentials.Token)
 
 	bodyProvider := common.BytesProvider{Data: content}
 
@@ -949,8 +913,6 @@ func (n *GitLabClient) UploadRawArtifacts(
 
 	query := uploadRawArtifactsQuery(options)
 
-	headers := make(http.Header)
-	headers.Set("JOB-TOKEN", config.Token)
 	res, err := n.doRaw(
 		context.Background(),
 		&config,
@@ -958,8 +920,7 @@ func (n *GitLabClient) UploadRawArtifacts(
 		fmt.Sprintf("jobs/%d/artifacts?%s", config.ID, query.Encode()),
 		bodyProvider,
 		contentType,
-		headers,
-	)
+		JobTokenHeader(config.Token))
 
 	defer func() { n.handleResponse(context.TODO(), res, true) }()
 
@@ -1079,11 +1040,16 @@ func (n *GitLabClient) DownloadArtifacts(
 		query.Set("direct_download", strconv.FormatBool(*directDownload))
 	}
 
-	headers := make(http.Header)
-	headers.Set("JOB-TOKEN", config.Token)
 	uri := fmt.Sprintf("jobs/%d/artifacts?%s", config.ID, query.Encode())
 
-	res, err := n.doRaw(context.Background(), &config, http.MethodGet, uri, nil, "", headers)
+	res, err := n.doRaw(
+		context.Background(),
+		&config,
+		http.MethodGet,
+		uri,
+		nil,
+		"",
+		JobTokenHeader(config.Token))
 
 	log := logrus.WithFields(logrus.Fields{
 		"id":    config.ID,
