@@ -2,7 +2,6 @@ package shells
 
 import (
 	"bytes"
-	"cmp"
 	"context"
 	"encoding/base64"
 	"fmt"
@@ -111,8 +110,6 @@ func NewPsWriter(b *PowerShell, info common.ShellScriptInfo) *PsWriter {
 	}
 }
 
-var encoder = unicode.UTF16(unicode.LittleEndian, unicode.IgnoreBOM).NewEncoder()
-
 func stdinCmdArgs(shell string, preCmds ...string) []string {
 	if shell == SNPwsh {
 		return pwshStdinCmdArgs(shell, preCmds...)
@@ -147,7 +144,7 @@ func pwshStdinCmdArgs(shell string, preCmds ...string) []string {
 	sb.WriteString("$OutputEncoding = [console]::InputEncoding = [console]::OutputEncoding = New-Object System.Text.UTF8Encoding\r\n")
 	sb.WriteString(shell + " -NoProfile -NonInteractive -Command -\r\n")
 	sb.WriteString("if(!$?) { Exit &{if($LASTEXITCODE) {$LASTEXITCODE} else {1}} }")
-	encoded, _ := encoder.String(sb.String())
+	encoded, _ := unicode.UTF16(unicode.LittleEndian, unicode.IgnoreBOM).NewEncoder().String(sb.String())
 
 	return append(
 		defaultPowershellFlags,
@@ -275,31 +272,45 @@ func (p *PsWriter) Command(command string, arguments ...string) {
 	p.checkErrorLevel()
 }
 
-// CommandWithStdin runs command with arguments and provides stdin to standard input stream of the command
-func (p *PsWriter) CommandWithStdin(stdin, command string, arguments ...string) {
-	// This mimics something like `echo "foobar" | blipp.exe` for pwsh/powershell, passing in stdin _as is_ to the
-	// command. It does not mess with the encoding, BOM, ...
-	// The unfortunate side-effect is, that we use a temporary file for this.
+// SetupGitCredHelper is the powershell implementation of setting up the runner's default cred helper, which pulls out
+// the job token from the environment.
+// For different editions/versions of powershell, we need to pass args differently, however we only have the version
+// information at runtime.
+func (p *PsWriter) SetupGitCredHelper(confFile, section, user string) {
+	helperSection := psSingleQuote(section + ".helper")
+	userSection := psSingleQuote(section + ".username")
+	confFile = p.resolvePath(confFile)
 
-	mainCommand := `Start-Process -NoNewWindow -RedirectStandardInput $tmpFile -Wait -FilePath ` + psSingleQuote(command)
-	if len(arguments) > 0 {
-		for i, arg := range arguments {
-			arguments[i] = psSingleQuote(arg)
-		}
-		mainCommand += " -ArgumentList " + strings.Join(arguments, ",")
-	}
+	clearCmd := fmt.Sprintf(`git config -f %s --replace-all %s`, confFile, helperSection)
+	setCmd := fmt.Sprintf(`git config -f %s --add %s`, confFile, helperSection)
+	userCmd := fmt.Sprintf(`git config -f %s %s %s`, confFile, userSection, psSingleQuote(user))
 
-	block := strings.Join([]string{
-		`try {`,
-		`$tmpFile = Get-Item ([System.IO.Path]::GetTempFilename())`,
-		fmt.Sprintf(`[System.IO.File]::WriteAllText($tmpFile, %s)`, psSingleQuote(stdin)),
-		mainCommand,
-		`} finally {`,
-		`if ($tmpFile) { Remove-Item -Force -Path $tmpFile }`,
+	// Specialities re. "special arguments" to external commands (empty args, args with `"`, ...):
+	//	- Starting from 7.3 we can pass on args without any special handling
+	//	- Before 7.3 we need to handle "special arguments" explicitly, ie. ensure we escape special stuff
+	//	- For 7.2.x it depends on the experimental feature PSNativeCommandArgumentPassing:
+	//		- if it's not enabled, we have to handle "special arguments" explicitly
+	//		- if it's enabled, we need to accept that; we can control exact behavior but we have to use it
+	// We run all that in a script block, so that setting PSNativeCommandArgumentPassing only effects this (and child)
+	// scopes.
+	p.Line(strings.Join([]string{
+		`& {`,
+		`$psVer = [Version]$PSVersionTable.PSVersion`,
+		`$needsSpecialArgQuoting = ($psVer -lt [Version]"7.3")`,
+		`if ( ($psVer -ge [Version]"7.2") -and ($psVer -lt [Version]"7.3") -and ((Get-ExperimentalFeature -Name PSNativeCommandArgumentPassing).Enabled) ) {`,
+		`  $PSNativeCommandArgumentPassing = 'Standard'`,
+		`  $needsSpecialArgQuoting = $False`,
 		`}`,
-	}, p.EOL)
-
-	p.Line(block)
+		`if ($needsSpecialArgQuoting) {`,
+		clearCmd + " " + `'""'`,
+		setCmd + " " + psSingleQuote(strings.ReplaceAll(credHelperCommand, `"`, `\"`)),
+		`} else {`,
+		clearCmd + " " + `''`,
+		setCmd + " " + psSingleQuote(credHelperCommand),
+		`}`,
+		userCmd,
+		`}`,
+	}, p.EOL))
 	p.CheckForErrors()
 }
 
@@ -431,6 +442,12 @@ func (p *PsWriter) IfCmdWithOutput(cmd string, arguments ...string) {
 	p.ifInTryCatch(p.buildCommand(psSingleQuote, cmd, arguments...))
 }
 
+func (p *PsWriter) IfGitVersionIsAtLeast(version string) {
+	p.Printf("Powershell does not support Git version detection")
+	p.Linef("if($false) {")
+	p.Indent()
+}
+
 func (p *PsWriter) ifInTryCatch(cmd string) {
 	p.Line("Set-Variable -Name cmdErr -Value $false")
 	p.Line("Try {")
@@ -644,56 +661,6 @@ func (b *PowerShell) GetName() string {
 	return b.Shell
 }
 
-const powershellGitCredHelperScript = `function f([string]$cmd){ if ($cmd.equals("get")) { Write-Host -NoNewline "password=${env:CI_JOB_TOKEN}` + "`n" + `" } }; f`
-
-// GetGitCredHelperCommand returns a command that can be used e.g. in a git config as a credential helper.
-//
-// This returns something like:
-//
-//	pwsh -NoProfile ... -Command ''function f{...}; ...; f''
-//
-// as a single string.
-//
-// Note the double single-quotes: This is deliberate!
-// This command is used with the shellwriter's Command(...), which will quote the whole string in single-quotes. We want
-// this to be a single argument (to `git config`) and not split into multiple arguments. Now that we know that the
-// result will be single-quoted again, we need to escape any "inner" single-quotes, and we do so doubling them here.
-//
-// In the end, for a successful configuration, we need the content of the git config to literally look something like:
-//
-//	[credential "https://gitlab.com"]
-//		username = gitlab-ci-token
-//		helper = "!pwsh -NoProfile -NoLogo -InputFormat text -OutputFormat text -NonInteractive -ExecutionPolicy Bypass -Command 'function f([string]$cmd){ if ($cmd.equals(\"get\")) { Write-Host -NoNewline \"password=${env:CI_JOB_TOKEN}`n\" } }; f'"
-//
-// or
-//
-//	[credential "https://gitlab.com"]
-//		username = gitlab-ci-token
-//		helper = "!powershell -NoProfile -NoLogo -InputFormat text -OutputFormat text -NonInteractive -ExecutionPolicy Bypass -Command 'function f([string]$cmd){ if ($cmd.equals(\"get\")) { Write-Host -NoNewline \"password=${env:CI_JOB_TOKEN}`n\" } }; f'"
-//
-// More docs about custom git cred helpers can be found at https://git-scm.com/docs/gitcredentials#_custom_helpers .
-func (b *PowerShell) GetGitCredHelperCommand(os string) string {
-	shell := b.GetName()
-	script := powershellGitCredHelperScript
-
-	// If the OS is not set explicitly, we fallback to the current processes' OS
-	os = cmp.Or(os, runtime.GOOS)
-
-	// Some special case for powershell on windows and weird quoting rules thereof.
-	// To be honest, I have no clue what's going on there, if this is a powershell thing, or if the shell writer
-	// interferes, or both; but it seems to be necessary and to work.
-	if shell == SNPowershell && os == OSWindows {
-		script = strings.ReplaceAll(script, `"`, `\"`)
-	}
-
-	return fmt.Sprintf(
-		"%s %s ''%s''",
-		shell,
-		strings.Join(append(defaultPowershellFlags, "-Command"), " "),
-		script,
-	)
-}
-
 func (b *PowerShell) GetEntrypointCommand(_ common.ShellScriptInfo, probeFile string) []string {
 	preCmds := []string{}
 	if probeFile != "" {
